/*-- 

 Copyright (C) 2001 Anthony Eden.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions, and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions, and the disclaimer that follows 
    these conditions in the documentation and/or other materials 
    provided with the distribution.

 3. The name "JPublish" must not be used to endorse or promote products
    derived from this software without prior written permission.  For
    written permission, please contact me@anthonyeden.com.
 
 4. Products derived from this software may not be called "JPublish", nor
    may "JPublish" appear in their name, without prior written permission
    from Anthony Eden (me@anthonyeden.com).
 
 In addition, I request (but do not require) that you include in the 
 end-user documentation provided with the redistribution and/or in the 
 software itself an acknowledgement equivalent to the following:
     "This product includes software developed by
      Anthony Eden (http://www.anthonyeden.com/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.

 For more information on JPublish, please see <http://www.jpublish.org/>.
 
 */

package org.jpublish;

import org.apache.velocity.VelocityContext;

/**	The JPublishContext overrides the Velocity context to add name
	checking as a security measure so that code cannot replace 
	standard variables.
	
	@author Anthony Eden
*/

public final class JPublishContext extends VelocityContext{

	public JPublishContext(Object owner){
		this.owner = owner;
	}

	/**	Put the given value into the context.  If the key cannot be
		inserted into the context because it conflicts with a reserved
		name then a ReservedNameException will be thrown.
	
		@param key The key
		@param value The value
		@return The previous value or null if there was no previous
			value
	*/

	public final Object put(String key, Object value){
		if(checkReservedNames && (isReservedName(key))){
			throw new ReservedNameException(key);
		}
			
		return super.put(key, value);
	}
	
	/**	Remove the given value from the context. If the key cannot be
		removed from the context because it is a reserved name then a 
		ReservedNameException will be thrown.
	
		@param key The key
		@return The removed value or null if there was no removed
			value
	*/
	
	public final Object remove(String key){
		if(checkReservedNames && isReservedName(key)){
			throw new ReservedNameException(key);
		}
		
		return super.remove(key);
	}
	
	/**	Enable the reserved name check so that external code cannot
		overwrite JPublish defined variables.
		
		@param owner The owner of the context.  Only the owner can
			
	*/
	
	public final void enableCheckReservedNames(Object owner){
		if(owner.equals(this.owner)){
			checkReservedNames = true;
		}
	}
	
	public final void disableCheckReservedNames(Object owner){
		if(owner.equals(this.owner)){
			checkReservedNames = false;
		}
	}
	
	/**	Return true if the given variable is a reserved name
		used by the JPublish engine. If checkReservedNames is
		true then this method must be called before inserting
		a value into the context.

		@param name The name to check
		@return True if the name is a reserved name
	*/
	
	private static final boolean isReservedName(String name){
		for(int i = 0; i < reservedNames.length; i++){
			if(name.equals(reservedNames[i])){
				return true;
			}
		}
		return false;
	}
	
	private static final String[] reservedNames = {
		"request", "response", "session", "page" };
	
	private Object owner;
	private boolean checkReservedNames = false;

}
