/*-- 

 Copyright (C) 2001 Anthony Eden.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions, and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions, and the disclaimer that follows 
    these conditions in the documentation and/or other materials 
    provided with the distribution.

 3. The name "JPublish" must not be used to endorse or promote products
    derived from this software without prior written permission.  For
    written permission, please contact me@anthonyeden.com.
 
 4. Products derived from this software may not be called "JPublish", nor
    may "JPublish" appear in their name, without prior written permission
    from Anthony Eden (me@anthonyeden.com).
 
 In addition, I request (but do not require) that you include in the 
 end-user documentation provided with the redistribution and/or in the 
 software itself an acknowledgement equivalent to the following:
     "This product includes software developed by
      Anthony Eden (http://www.anthonyeden.com/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.

 For more information on JPublish, please see <http://www.jpublish.org/>.
 
 */

package org.jpublish;

import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Writer;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import org.apache.velocity.VelocityContext;

import com.anthonyeden.lib.log.Logger;
import com.anthonyeden.lib.log.LogManager;

import org.jpublish.action.Action;
import org.jpublish.action.ScriptAction;

/**	A representation of a single web page.  A page is defined by an XML
	document in the pages directory.  Each page has a template associated
	with the page and can have 0 or more actions attached to the page. 
	
	<p>Actions attached to a page will be triggered each time the page
	is requested.  Actions will be triggered in the order that they are
	listed within the Page's configuration.</p>

	@author Anthony Eden
*/

public class Page implements XMLConstants{

	/**	Construct a new Page for the given path.  The name of the page is 
		the last part of the path (i.e. the file component) without the 
		dot ending.  The page type is the dot-ending.
	
		@param siteContext The SiteContext
		@param path The request path
		@param pageName The name of the page
		@param pageType The page type
	*/
		
	public Page(SiteContext siteContext, String path, String pageName, String pageType){
		this.siteContext = siteContext;
		this.path = path;
		this.pageName = pageName;
		this.pageType = pageType;
		this.pageActions = new ArrayList();
		this.templateName = siteContext.getDefaultTemplate();
		this.ejbs = new HashMap();
		this.exposedEjbs = new HashMap();
	}
	
	/**	Get the request path.
	
		@return The request path
	*/
	
	public String getPath(){
		return path;
	}
	
	/**	Get the page name.
	
		@return The page name
	*/
	
	public String getPageName(){
		return pageName;
	}
	
	/**	Get the page type.
	
		@return The page type
	*/
	
	public String getPageType(){
		return pageType;
	}
	
	/**	Return the page title.  Initially the page title is extracted from 
		the page's definition document, however it can be set programtically
		at runtime.
	
		@return The page title
	*/

	public String getTitle(){
		return title;
	}
	
	/**	Set the title.  This will temporarily alter the page's title.
	
		@param title The page title
	*/
	
	public void setTitle(String title){
		this.title = title;
	}
	
	/**	Get the full template file name, with the .suffix attached.
	
		@return The full template name
	*/
	
	public String getFullTemplateName(){
		return templateName + "." + pageType;
	}
	
	/**	Get the template name.  If the template name is not specified in the
		page configuration then the default template as specified in the 
		SiteContext will be used.
	
		@return The template name
	*/
	
	public String getTemplateName(){
		return templateName;
	}
	
	/**	Set the template name.  Invoking this method with a null value will 
		reset the template to the default template as specified in the 
		SiteContext.
	
		@param templateName The new template name or null to reset
	*/
	
	public synchronized void setTemplateName(String templateName){
		if(templateName == null){
			templateName = siteContext.getDefaultTemplate();
		}
		this.templateName = templateName;
	}
	
	/**	Get a List of page actions.  To add an action to the page just add
		the action to this List.  Page actions are triggered each time the
		page is requested.
		
		@return A List of page actions
	*/
	
	public List getPageActions(){
		return pageActions;
	}
	
	/**	Get a Map of EJBHome interfaces.
	
		@return A Map of name/EJBHome pairs
	*/
	
	public Map getEJBs(){
		return ejbs;
	}
	
	/**	Get a Map of EJB Session beans which are exposed on the page
		directly.
		
		@return A Map of name/session bean pairs
	*/
	
	public Map getExposedEJBs(){
		return exposedEjbs;
	}
	
	/**	Execute the page actions using the given VelocityContext.
	
		@param context The VelocityContext
		@return A redirection value or null if there is no redirection
		@throws Exception Any Exception which occurs while executing the action
	*/
	
	public String executeActions(VelocityContext context) throws Exception{
		Iterator pageActions = getPageActions().iterator();
		while(pageActions.hasNext()){
			((Action)pageActions.next()).execute(context);
			
			String redirect = (String)context.get("redirect");
			if(redirect != null){
				return redirect;
			}
		}
		return null;
	}
	
	/**	Load the page configuration from the page's XML stream.
	
		@param in The InputStream
		@throws Exception Any exception
	*/
	
	public void load(InputStream in) throws Exception{
		log.debug("Loading page.");
		SAXBuilder builder = new SAXBuilder();
		Document document = builder.build(in);
		Element rootElement = document.getRootElement();
		
		// set the title
		String title = rootElement.getChildTextTrim("title", NS);
		setTitle(title);
		log.debug("Set title: " + title);
		
		// set the template name
		String template = rootElement.getChildTextTrim("template", NS);
		setTemplateName(template);
		log.debug("Set template: " + template);
		
		Map definedActions = siteContext.getActionManager().getDefinedActions();
		log.debug("Retrieved defined actions.");
		
		// load page actions
		log.debug("Looping through page-action elements.");
		Iterator pageActionElements = rootElement.getChildren("page-action", NS).iterator();
		while(pageActionElements.hasNext()){
			Element pageActionElement = (Element)pageActionElements.next();
			String name = pageActionElement.getAttributeValue("name");
			if(name == null){
				name = pageActionElement.getTextTrim();
			}
			
			if(name == null){
				throw new Exception("Error configuring page-action.");
			}
			
			if(definedActions.containsKey(name)){
				pageActions.add(definedActions.get(name));
			} else {
				File script = new File(siteContext.getActionRoot(), name);
				pageActions.add(new ScriptAction(siteContext, script));
			}
		}
		
		// load EJB elements
		log.debug("Loading EJB elements");
		Iterator ejbElements = rootElement.getChildren("ejb", NS).iterator();
		while(ejbElements.hasNext()){
			Element ejbElement = (Element)ejbElements.next();
			String name = ejbElement.getAttributeValue("name");
			String ref = ejbElement.getChildTextTrim("ref");
			String homeClass = ejbElement.getChildTextTrim("home");
			
			log.debug("EJB Eelement: " + name);
		
			EJB ejb = new EJB(ref, homeClass);
			ejbs.put(name, ejb);
		}
		
		// expose session beans.  The home interface for the session
		// bean must have already been specified.
		log.debug("Loading exposed EJBs");
		Iterator exposeEjbElements = rootElement.getChildren("expose-ejb", NS).iterator();
		while(exposeEjbElements.hasNext()){
			log.debug("EJB session bean found.");
			Element exposeEjbElement = (Element)exposeEjbElements.next();
			String name = exposeEjbElement.getAttributeValue("name");
			String home = exposeEjbElement.getAttributeValue("home");
			log.debug("Found ejb. [name=" + name + ",home=" + home + "]");
			
			if(ejbs.containsKey(home) || siteContext.getEJBs().containsKey(home)){
				log.debug("Putting into exposed EJB [name=" + name + ",home=" + home + "]");
				exposedEjbs.put(name, home);
			}
		}
	}
	
	private static Logger log = LogManager.getLogger(Page.class.getName());
	
	private SiteContext siteContext;
	private List pageActions;
	private String path;
	private String pageName;
	private String pageType;
	private String title;
	private String templateName;
	private Map ejbs;
	private Map exposedEjbs;

}
