/*-- 

 Copyright (C) 2001 Anthony Eden.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions, and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions, and the disclaimer that follows 
    these conditions in the documentation and/or other materials 
    provided with the distribution.

 3. The name "JPublish" must not be used to endorse or promote products
    derived from this software without prior written permission.  For
    written permission, please contact me@anthonyeden.com.
 
 4. Products derived from this software may not be called "JPublish", nor
    may "JPublish" appear in their name, without prior written permission
    from Anthony Eden (me@anthonyeden.com).
 
 In addition, I request (but do not require) that you include in the 
 end-user documentation provided with the redistribution and/or in the 
 software itself an acknowledgement equivalent to the following:
     "This product includes software developed by
      Anthony Eden (http://www.anthonyeden.com/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.

 For more information on JPublish, please see <http://www.jpublish.org/>.
 
 */

package org.jpublish;

import java.io.File;

import java.io.InputStream;
import java.io.FileInputStream;

import java.util.List;
import java.util.Map;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.TreeMap;
import java.util.Iterator;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;

import org.formproc.FormManager;

import com.anthonyeden.lib.Reloadable;
import com.anthonyeden.lib.log.Logger;
import com.anthonyeden.lib.log.LogManager;
import com.anthonyeden.lib.util.IOUtilities;
import com.anthonyeden.lib.util.FileMonitor;
import com.anthonyeden.lib.util.LogUtilities;

import org.jpublish.util.MimeTypeMap;
import org.jpublish.action.Action;
import org.jpublish.action.ScriptAction;
import org.jpublish.action.PathAction;
import org.jpublish.action.ActionManager;

/**	This class contains configuration information for a particular
	site.  Pages are loaded and actions are executed within the
	context of the site and have access to all of the methods within
	this class.
	
	<p>Instances of the SiteContext class will also reload themselves
	automatically when the underlying configuration file changes.</p>
	
	@author Anthony Eden
*/

public class SiteContext implements XMLConstants, Reloadable{

	/**	Construct a new SiteContext using the given File to load the
		context's configuration.
		
		@param configurationFile The configuration File
	*/

	public SiteContext(File configurationFile){
		this.configurationFile = configurationFile;
		
		repositories = new ArrayList();
		ejbs = new TreeMap();
		exposedEjbs = new TreeMap();
		mimeTypeMap = new MimeTypeMap();
		staticResourceLoader = new StaticResourceLoader();
		
		reload();
		
		FileMonitor fileMonitor = new FileMonitor(this.configurationFile, 5000, this);
		fileMonitor.startMonitor();
	}
	
	/**	Get the directory where page configuration files are stored.  Page
		configurations are stored as XML files and are cached to improve
		performance.  The page will automatically be reloaded if it is modified.
		
		@return The page root
	*/

	public File getPageRoot(){
		return pageRoot;
	}
	
	/**	Set the directory where page configuration files are stored.  Page
		configurations are stored as XML files and are cached to improve
		performance.  The page will automatically be reloaded if it is modified.
		
		@param pageRoot The new page root
	*/
	
	public void setPageRoot(File pageRoot){
		this.pageRoot = pageRoot;
	}
	
	/**	Set the directory where page configuration files are stored.  Page
		configurations are stored as XML files and are cached to improve
		performance.  The page will automatically be reloaded if it is modified.
		
		@param pageRoot The new page root
	*/
	
	public void setPageRoot(String pageRoot){
		if(pageRoot != null){
			setPageRoot(new File(pageRoot));
		}
	}
	
	/**	Get the directory where templates are stored.  Templates are merged
		with the runtime context to produce a final document.
		
		@return The template root
	*/
	
	public File getTemplateRoot(){
		return templateRoot;
	}
	
	/**	Set the directory where templates are stored.  Templates are merged
		with the runtime context to produce a final document.
		
		@param templateRoot The template root
	*/
	
	public void setTemplateRoot(File templateRoot){
		this.templateRoot = templateRoot;
	}
	
	/**	Set the directory where templates are stored.  Templates are merged
		with the runtime context to produce a final document.
		
		@param templateRoot The template root
	*/
	
	public void setTemplateRoot(String templateRoot){
		if(templateRoot != null){
			setTemplateRoot(new File(templateRoot));
		}
	}
	
	/**	Get the directory where action scripts are stored.  Actions stored
		here are scripts written in a language supported by the BSF library.
		The language's Java implementation must be included in the classpath.
	
		@return The action root
	*/
	
	public File getActionRoot(){
		return actionRoot;
	}
	
	/**	Set the directory where action scripts are stored.  Actions stored
		here are scripts written in a language supported by the BSF library.
		The language's Java implementation must be included in the classpath.
	
		@param actionRoot The new action root
	*/
	
	public void setActionRoot(File actionRoot){
		this.actionRoot = actionRoot;
	}
	
	/**	Set the directory where action scripts are stored.  Actions stored
		here are scripts written in a language supported by the BSF library.
		The language's Java implementation must be included in the classpath.
	
		@param actionRoot The new action root
	*/
	
	public void setActionRoot(String actionRoot){
		if(actionRoot != null){
			setActionRoot(new File(actionRoot));
		}
	}
	
	/**	Get the directory where static files are stored.  Static files are
		files which are not processed by JPublish but are returned byte for
		byte.
		
		@return The root directory where static files are stored
	*/
	
	public File getStaticRoot(){
		return staticRoot;
	}
	
	/**	Set the directory where static files are stored.  Static files are
		files which are not processed by JPublish but are returned byte for
		byte.
		
		@param staticRoot The new root directory where static files are stored
	*/
	
	public void setStaticRoot(File staticRoot){
		this.staticRoot = staticRoot;
	}
	
	/**	Set the directory where static files are stored.  Static files are
		files which are not processed by JPublish but are returned byte for
		byte.
		
		@param staticRoot The path of the new root directory where static 
			files are stored
	*/
	
	public void setStaticRoot(String staticRoot){
		if(staticRoot != null){
			setStaticRoot(new File(staticRoot));
		}
	}
	
	/**	Get the static resource loader.
	
		@return The static resource loader
		@see StaticResourceLoader
	*/
	
	public StaticResourceLoader getStaticResourceLoader(){
		return staticResourceLoader;
	}
	
	/**	Get the File for the WEB-INF directory.  This is used to locate the
		classes and JARs so they are accessible to scripting languages.
		
		@return The WEB-INF file
	*/
	
	public File getWebInfPath(){
		return webInfPath;
	}
	
	/**	Set the File for the WEB-INF directory.
		
		@param webInfPath The WEB-INF file
	*/
	
	public void setWebInfPath(File webInfPath){
		this.webInfPath = webInfPath;
	}
	
	/**	Get the action identifier which is used to trigger parameter actions.
		The default value is 'action'.
	
		@return The action identifier
	*/
	
	public String getActionIdentifier(){
		return actionIdentifier;
	}
	
	/**	Set the action identifier.  If this method is invoked with a null
		argument then the action identifier will be reset to the default
		value.
		
		@param actionIdentifier The new action identifer or null to reset
	*/
	
	public synchronized void setActionIdentifier(String actionIdentifier){
		if(actionIdentifier == null){
			this.actionIdentifier = DEFAULT_ACTION_INDENTIFIER;
		} else {
			this.actionIdentifier = actionIdentifier;
		}
	}
	
	/**	Get the default page.  This value will be used when directories
		are requested.  By default this returns 'index.html'.
		
		@return The default page
	*/
	
	public String getDefaultPage(){
		return defaultPage;
	}
	
	/**	Set the default page.  This value will be used when directories
		are requested.
	
		@param defaultPage The new default page
	*/
	
	public void setDefaultPage(String defaultPage){
		this.defaultPage = defaultPage;
	}
	
	/**	Get the default template.  This method returns the name of the
		template which will be used when no template is specified in 
		a page's configuration.
		
		@return The default template
	*/
	
	public String getDefaultTemplate(){
		return defaultTemplate;
	}
	
	/**	Set the default template. The default template will be used when 
		no template is specified in a page's configuration.
		
		@param defaultTemplate The new default template
	*/
	
	public void setDefaultTemplate(String defaultTemplate){
		this.defaultTemplate = defaultTemplate;
	}
	
	/**	Get the default mime type.  This method delegates to the current
		MimeTypeMap.
		
		@return The default mime type
	*/
	
	public String getDefaultMimeType(){
		return getMimeTypeMap().getDefaultMimeType();
	}
	
	/**	Set the default mime type.  This method delegates to the current
		MimeTypeMap.
		
		@param defaultMimeType The new default mime type
	*/
	
	public void setDefaultMimeType(String defaultMimeType){
		getMimeTypeMap().setDefaultMimeType(defaultMimeType);
	}
	
	/**	Get a list of all registered repositories.
	
		@return A list of all registered Repository objects
	*/
	
	public List getRepositories(){
		return repositories;
	}
	
	/**	Get a Map of EJBHome interfaces.
	
		@return A Map of name/EJBHome pairs
	*/
	
	public Map getEJBs(){
		return ejbs;
	}
	
	/**	Get a Map of EJB Session beans which are exposed on the page
		directly.
		
		@return A Map of name/session bean pairs
	*/
	
	public Map getExposedEJBs(){
		return exposedEjbs;
	}
	
	/**	Get a Repository by name.  If the repository was not registered
		then this method will return null.
		
		@param name The name of the Repository
		@return The repository or null
	*/
	
	public Repository getRepository(String name){
		Iterator iter = getRepositories().iterator();
		while(iter.hasNext()){
			Repository repository = (Repository)iter.next();
			if(repository.getName().equals(name)){
				return repository;
			}
		}
		return null;
	}
	
	/**	Get the site's ActionManager.
	
		@return The ActionManager
		@see org.jpublish.action.ActionManager
	*/
	
	public ActionManager getActionManager(){
		return actionManager;
	}
	
	/**	Get the site's PageManager.
	
		@return The PageManager
		@see org.jpublish.PageManager
	*/
	
	public PageManager getPageManager(){
		return pageManager;
	}
	
	/**	Get the site's FormManager.  If no FormManager is specified then
		this method will return null.
	
		@return The FormManager
		@see org.formproc.FormManager
	*/
	
	public FormManager getFormManager(){
		return formManager;
	}
	
	/**	Get the site's MimeType map.  Mime types can be mapped to file
		suffixes.
		
		@return The MimeTypeMap
	*/
	
	public MimeTypeMap getMimeTypeMap(){
		return mimeTypeMap;
	}
	
	/**	Reload the site configuration. */
	
	public void reload(){
		try{
			log.info("Loading site configuration.");
			loadConfiguration();
			log.info("Configuration loaded.");
		} catch(Exception e){
			e.printStackTrace();
		}
	}
	
	/**	Load the site configuration from the given InputStream.  The
		InputStream must be attached to an XML document.
		
		@param in The InputStream
		@throws Exception
	*/

	public void load(InputStream in) throws Exception{
		actionManager = new ActionManager(this);
		
		pageManager = (PageManager)Class.forName(
			"org.jpublish.page.filesystem.FileSystemPageManager").newInstance();
		pageManager.setSiteContext(this);
	
		SAXBuilder builder = new SAXBuilder();
		Document document = builder.build(in);
		Element rootElement = document.getRootElement();
		
		setPageRoot(rootElement.getChildTextTrim("page-root", NS));
		setTemplateRoot(rootElement.getChildTextTrim("template-root", NS));
		setActionRoot(rootElement.getChildTextTrim("action-root", NS));
		setStaticRoot(rootElement.getChildTextTrim("static-root", NS));
		
		createFormManager(rootElement.getChildTextTrim("formproc-config", NS));
		
		setActionIdentifier(rootElement.getChildTextTrim("action-identifier", NS));
		
		setDefaultPage(rootElement.getChildTextTrim("default-page", NS));
		setDefaultTemplate(rootElement.getChildTextTrim("default-template", NS));
		setDefaultMimeType(rootElement.getChildTextTrim("default-mime-type", NS));
		
		configureJNDI(rootElement.getChild("jndi", NS));
		
		// load EJB elements
		log.debug("Loading EJB elements");
		Iterator ejbElements = rootElement.getChildren("ejb", NS).iterator();
		while(ejbElements.hasNext()){
			Element ejbElement = (Element)ejbElements.next();
			String name = ejbElement.getAttributeValue("name");
			String ref = ejbElement.getChildTextTrim("ref");
			String homeClass = ejbElement.getChildTextTrim("home");
			
			log.debug("EJB Eelement: " + name);
		
			EJB ejb = new EJB(ref, homeClass);
			ejbs.put(name, ejb);
		}
		
		// expose session beans.  The home interface for the session
		// bean must have already been specified.
		log.debug("Loading exposed EJBs");
		Iterator exposeEjbElements = rootElement.getChildren("expose-ejb", NS).iterator();
		while(exposeEjbElements.hasNext()){
			log.debug("EJB session bean found.");
			Element exposeEjbElement = (Element)exposeEjbElements.next();
			String name = exposeEjbElement.getAttributeValue("name");
			String home = exposeEjbElement.getAttributeValue("home");
			log.debug("Found ejb. [name=" + name + ",home=" + home + "]");
			
			if(ejbs.containsKey(home)){
				log.debug("Putting into exposed EJB [name=" + name + ",home=" + home + "]");
				exposedEjbs.put(name, home);
			}
		}
		
		// load repository references
		Iterator repositoryElements = rootElement.getChildren("repository", NS).iterator();
		while(repositoryElements.hasNext()){
			Element repositoryElement = (Element)repositoryElements.next();
			String className = repositoryElement.getAttributeValue("classname");
			Repository repository = (Repository)Class.forName(className).newInstance();
			repository.setSiteContext(this);
			repository.loadConfiguration(repositoryElement);
			getRepositories().add(repository);
		}
		
		// load defined actions
		Map definedActions = actionManager.getDefinedActions();
		Iterator defineActionElements = rootElement.getChildren("define-action", NS).iterator();
		while(defineActionElements.hasNext()){
			Element defineActionElement = (Element)defineActionElements.next();
			String name = defineActionElement.getAttributeValue("name");
			String className = defineActionElement.getAttributeValue("classname");
			Action action = (Action)Class.forName(className).newInstance();
			definedActions.put(name, action);
		}
		
		// load startup action configurations
		List startupActions = actionManager.getStartupActions();
		Iterator startupActionElements = rootElement.getChildren("startup-action", NS).iterator();
		while(startupActionElements.hasNext()){
			Element startupActionElement = (Element)startupActionElements.next();
			String name = startupActionElement.getAttributeValue("name");
			startupActions.add(actionManager.findAction(name));
		}
		
		// load global action configurations
		List globalActions = actionManager.getGlobalActions();
		Iterator globalActionElements = rootElement.getChildren("global-action", NS).iterator();
		while(globalActionElements.hasNext()){
			Element globalActionElement = (Element)globalActionElements.next();
			String name = globalActionElement.getAttributeValue("name");
			globalActions.add(actionManager.findAction(name));
		}
		
		// load path action configurations
		List pathActions = actionManager.getPathActions();
		Iterator pathActionElements = rootElement.getChildren("path-action", NS).iterator();
		while(pathActionElements.hasNext()){
			Element pathActionElement = (Element)pathActionElements.next();
			String name = pathActionElement.getAttributeValue("name");
			String path = pathActionElement.getAttributeValue("path");
			
			// debugging
			HashMap data = new HashMap();
			data.put("path", path);
			data.put("action", name);
			LogUtilities.debug(log, "Registered path action.", data);
			
			pathActions.add(new PathAction(path, actionManager.findAction(name)));
		}
		
		// load the mime type map
		Iterator mimeTypeMapElements = rootElement.getChildren("mime-mapping", NS).iterator();
		while(mimeTypeMapElements.hasNext()){
			Element mimeTypeMapElement = (Element)mimeTypeMapElements.next();
			String ext = mimeTypeMapElement.getAttributeValue("ext");
			String mimeType = mimeTypeMapElement.getAttributeValue("mimetype");
			
			// debugging
			HashMap data = new HashMap();
			data.put("ext", ext);
			data.put("mimetype", mimeType);
			LogUtilities.debug(log, "Registered mime type.", data);
			
			mimeTypeMap.put(ext, mimeType);
		}
	}
	
	// private methods
	
	/**	Configure the initial JNDI context properties.  If the element is null
		then the property values will not be modified.
		
		@param element The JDOM configuration element
		@throws Exception
	*/
	
	private void configureJNDI(Element element) throws Exception{
		if(element != null){
			System.setProperty("java.naming.factory.initial", 
				element.getChildTextTrim("initial-factory"));
			System.setProperty("java.naming.provider.url", 
				element.getChildTextTrim("provider"));
		}
	}
	
	/**	Load the configuration.  This method opens the stream to the
		configuration and then calls the <code>load()</code> method.
		
		@throws Exception
	*/
	
	private void loadConfiguration() throws Exception{
		InputStream in = null;
		
		try{
			in = new FileInputStream(configurationFile);
			load(in);
		} catch(Exception e){
			throw e;
		} finally {
			IOUtilities.close(in);
		}
	}
	
	/**	Create a FormManager using the given configuration file.
	
		@param configFile The FormManager configuration file
		@throws Exception Any Exception
	*/
	
	private void createFormManager(String configFile) throws Exception{
		if(configFile != null){
			formManager = new FormManager();
			formManager.loadConfiguration(new File(configFile));
		}
	}
	
	private static final String DEFAULT_ACTION_INDENTIFIER = "action";
	
	private static Logger log = LogManager.getLogger(SiteContext.class.getName());
	
	private File configurationFile;
	
	private File pageRoot;
	private File templateRoot;
	private File actionRoot;
	private File staticRoot;
	private File webInfPath;
	private String actionIdentifier;
	private String defaultPage = "index.html";
	private String defaultTemplate = "basic";
	private List repositories;
	private Map ejbs;
	private Map exposedEjbs;
	
	private PageManager pageManager;
	private ActionManager actionManager;
	private StaticResourceLoader staticResourceLoader;
	private FormManager formManager;
	private MimeTypeMap mimeTypeMap;

}
