/*-- 

 Copyright (C) 2001 Anthony Eden.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions, and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions, and the disclaimer that follows 
    these conditions in the documentation and/or other materials 
    provided with the distribution.

 3. The name "JPublish" must not be used to endorse or promote products
    derived from this software without prior written permission.  For
    written permission, please contact me@anthonyeden.com.
 
 4. Products derived from this software may not be called "JPublish", nor
    may "JPublish" appear in their name, without prior written permission
    from Anthony Eden (me@anthonyeden.com).
 
 In addition, I request (but do not require) that you include in the 
 end-user documentation provided with the redistribution and/or in the 
 software itself an acknowledgement equivalent to the following:
     "This product includes software developed by
      Anthony Eden (http://www.anthonyeden.com/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.

 For more information on JPublish, please see <http://www.jpublish.org/>.
 
 */

package org.jpublish.page.filesystem;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.HashMap;

import com.anthonyeden.lib.log.Logger;
import com.anthonyeden.lib.log.LogManager;
import com.anthonyeden.lib.util.IOUtilities;

import org.jpublish.Page;
import org.jpublish.page.PageCacheEntry;
import org.jpublish.page.AbstractPageManager;

/**	The PageManager is a central access point for locating pages.  Pages
	are loaded and cached automatically.  The cache will check the file's
	last modification time and will update if the stored time does not match
	the file system's time.

	@author Anthony Eden
*/

public class FileSystemPageManager extends AbstractPageManager{

	/**	Construct a new PageManager using the given SiteContext.
	
		@param siteContext The SiteContext
	*/

	public FileSystemPageManager(){
		this.cache = new HashMap();
	}
	
	/**	Get the root directory for pages.  Page requests are appended
		to this root directory to locate the page's XML configuration
		file.
	
		@return The root directory for pages
	*/

	public File getRoot(){
		return siteContext.getPageRoot();
	}
	
	/**	Get a Page instance from the given path.  If no page can be found
		then this method will throw a FileNotFoundException.
	
		@param path The page path
		@return The Page
		@throws Exception Any Exception
	*/

	public synchronized Page getPage(String path) throws Exception{
		File file = new File(path);
		File parentDirectory = file.getParentFile();
		String fileName = file.getName();
		int dotIndex = fileName.lastIndexOf(".");
		String pageName = fileName.substring(0, dotIndex);
		String pageType = fileName.substring(dotIndex + 1);
		log.debug("Page name: " + pageName);
		log.debug("Page type: " + pageType);
		
		File xmlFile = new File(getRoot(), 	
			new File(parentDirectory, pageName + ".xml").getPath());
		
		if(!xmlFile.exists()){
			throw new FileNotFoundException("File not found: " + xmlFile);
		}
		
		log.debug("Looking for page:" + xmlFile);
		
		// check the cache and load the page if necessary
		PageCacheEntry cacheEntry = (PageCacheEntry)cache.get(path);
		Page page = null;
		
		FileInputStream in = null;
		try{
			if(cacheEntry == null){
				log.debug("Page (" + path + ") not found in cache.");
				page = new Page(siteContext, path, pageName, pageType);
				log.debug("Loading page: " + xmlFile);
				in = new FileInputStream(xmlFile);
				page.load(in);
				cache.put(path, new PageCacheEntry(page, xmlFile.lastModified()));
			} else {
				log.debug("Page (" + path + ") found in cache.");
				page = cacheEntry.getPage();
				if(cacheEntry.getLastModified() != xmlFile.lastModified()){
					log.debug("Page modification dates do not match.  Reloading page.");
					long lastModified = xmlFile.lastModified();
					in = new FileInputStream(xmlFile);
					page.load(in);
					cacheEntry.setLastModified(lastModified);
				}
			}
		} catch(Exception e){
			throw e;
		} finally {
			IOUtilities.close(in);
		}
		
		return page;
	}
	
	private static Logger log = LogManager.getLogger(FileSystemPageManager.class.getName());
	
	private File root;
	private HashMap cache;

}
