/*--

 Copyright (C) 2001 Anthony Eden.
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions, and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions, and the disclaimer that follows
    these conditions in the documentation and/or other materials
    provided with the distribution.

 3. The name "JPublish" must not be used to endorse or promote products
    derived from this software without prior written permission.  For
    written permission, please contact me@anthonyeden.com.

 4. Products derived from this software may not be called "JPublish", nor
    may "JPublish" appear in their name, without prior written permission
    from Anthony Eden (me@anthonyeden.com).

 In addition, I request (but do not require) that you include in the
 end-user documentation provided with the redistribution and/or in the
 software itself an acknowledgement equivalent to the following:
     "This product includes software developed by
      Anthony Eden (http://www.anthonyeden.com/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

 For more information on JPublish, please see <http://www.jpublish.org/>.

 */

package org.jpublish.repository.db;

import java.io.StringReader;
import java.io.StringWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;

import org.jdom.Element;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

import com.anthonyeden.lib.log.Logger;
import com.anthonyeden.lib.log.LogManager;
import com.anthonyeden.lib.util.IOUtilities;

import org.jpublish.repository.AbstractRepository;

public class DBRepository extends AbstractRepository{

	/**	Get the name of the repository.  This name is used to expose the
		Repository in the Velocity engine.

		@return The Repository name
	*/

	public String getName(){
		return name;
	}

	/**	Get the content from the given path.  Implementations of this method
		should NOT merge the content using Velocity.

		@param path The relative content path
		@return The content as a String
		@throws Exception Any Exception
	*/

	public String get(String path) throws Exception{
		log.debug("Getting static content element.");
	
		Connection c = null;
		try{
			log.debug("Connecting to database.");
			c = DriverManager.getConnection(url, username, password);
			
			PreparedStatement s = c.prepareStatement(contentQuery);
			s.setString(1, path);
			
			log.debug("Executing query.");
			ResultSet r = s.executeQuery();
			if(r.next()){
				return r.getString(1);
			} else {
				throw new Exception("Path not found: " + path);
			}
		} finally {
			c.close();
		}
	}
	
	/**	Get the content from the given path and merge it with
		the given context.
		
		@param path The content path
		@param context The VelocityContext
		@return The content as a String
		@throws Exception Any Exception
	*/

	public String get(String path, VelocityContext context) throws Exception{
		log.debug("Getting dynamic content element for path " + path);
		StringWriter writer = null;
		StringReader reader = null;
		
		try{
			writer = new StringWriter();
			reader = new StringReader(get(path));
		
			Velocity.evaluate(context, writer, path, reader);
			return writer.toString();
		} finally {
			IOUtilities.close(writer);
			IOUtilities.close(reader);
		}
	}

	/**	Get the last modified time in milliseconds for the given path.

		@param path The content path
		@return The last modified time in milliseconds
		@throws Exception Any exception
	*/

	public long getLastModified(String path) throws Exception{
		Connection c = null;
		try{
			log.debug("Connecting to database.");
			c = DriverManager.getConnection(url, username, password);
			
			PreparedStatement s = c.prepareStatement(lastModifiedQuery);
			s.setString(1, path);
			
			log.debug("Executing query.");
			ResultSet r = s.executeQuery();
			if(r.next()){
				return r.getLong(1);
			} else {
				throw new Exception("Path not found: " + path);
			}
		} finally {
			c.close();
		}
	}
	
	/**	Load the repository's configuration from the given JDOM element.
	
		@param element The JDOM element
		@throws Exception
	*/
	
	public void loadConfiguration(Element element) throws Exception{
		this.name = element.getAttributeValue("name");
		setURL(element.getChildTextTrim("url"));
		setUsername(element.getChildTextTrim("username"));
		setPassword(element.getChildTextTrim("password"));
		
		setContentQuery(element.getChildTextTrim("content-query"));
		setLastModifiedQuery(element.getChildTextTrim("last-modified-query"));
	}
	
	protected void setURL(String url){
		this.url = url;
	}
	
	protected void setUsername(String username){
		this.username = username;
	}
	
	protected void setPassword(String password){
		this.password = password;
	}
	
	protected void setContentQuery(String contentQuery){
		this.contentQuery = contentQuery;
	}
	
	protected void setLastModifiedQuery(String lastModifiedQuery){
		this.lastModifiedQuery = lastModifiedQuery;
	}
	
	private static final Logger log = LogManager.getLogger(DBRepository.class.getName());

	private String name;

	private String url;
	private String username;
	private String password;
	
	private String contentQuery;
	private String lastModifiedQuery;

}
