/*-- 

 Copyright (C) 2001 Anthony Eden.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions, and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions, and the disclaimer that follows 
    these conditions in the documentation and/or other materials 
    provided with the distribution.

 3. The name "JPublish" must not be used to endorse or promote products
    derived from this software without prior written permission.  For
    written permission, please contact me@anthonyeden.com.
 
 4. Products derived from this software may not be called "JPublish", nor
    may "JPublish" appear in their name, without prior written permission
    from Anthony Eden (me@anthonyeden.com).
 
 In addition, I request (but do not require) that you include in the 
 end-user documentation provided with the redistribution and/or in the 
 software itself an acknowledgement equivalent to the following:
     "This product includes software developed by
      Anthony Eden (http://www.anthonyeden.com/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.

 For more information on JPublish, please see <http://www.jpublish.org/>.
 
 */

package org.jpublish.repository.filesystem;

import java.io.File;
import java.io.FileReader;
import java.io.StringWriter;
import java.util.Iterator;

import org.jdom.Element;
import org.jdom.Document;
import org.jdom.input.SAXBuilder;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

import com.anthonyeden.lib.log.Logger;
import com.anthonyeden.lib.log.LogManager;
import com.anthonyeden.lib.util.IOUtilities;

import org.jpublish.Configurable;
import org.jpublish.XMLConstants;
import org.jpublish.action.ActionManager;
import org.jpublish.repository.AbstractRepository;

/**	The ExtendedFileSystemRepository allows actions to be bound to content
	elements through XML configuration files.  Only actions bound to dynamic 
	content elements will be executed.
	
	<p>Note: Actions attached to content elements cannot cause an HTTP redirect.
	The "redirect" value will be ignored.</p>
	
	@author Anthony Eden
*/

public class ExtendedFileSystemRepository extends AbstractRepository implements 
Configurable, XMLConstants{
	
	public String get(String path) throws Exception{
		log.debug("Getting static content element.");
		return loadContent(path);
	}

	public String get(String path, VelocityContext context) throws Exception{
		executeActions(path, context);
		
		log.debug("Getting dynamic content element for path " + path);
		
		StringWriter writer = null;
		FileReader reader = null;
		
		try{
			reader = new FileReader(pathToFile(path));
			Velocity.evaluate(context, writer, path, reader);
			String content = writer.toString();
			return content;
		} finally {
			IOUtilities.close(reader);
			IOUtilities.close(writer);
		}
	}
	
	public String getName(){
		return name;
	}
	
	public long getLastModified(String path){
		return pathToFile(path).lastModified();
	}
	
	public String getRoot(){
		return root;
	}
	
	public void setRoot(String root){
		this.root = root;
	}
	
	public void loadConfiguration(Element element) throws Exception{
		this.name = element.getAttributeValue("name");
		setRoot(element.getChildTextTrim("root"));
	}
	
	private String loadContent(String path){
		
		StringBuffer buffer = new StringBuffer();
		FileReader reader = null;
		
		try{
			reader = new FileReader(pathToFile(path));
			
			int c = -1;
			while((c = reader.read()) != -1){
				buffer.append((char)c);
			}
		} catch(Exception e){
			buffer.append(e.getMessage());
		} finally {
			IOUtilities.close(reader);
		}
		
		return buffer.toString();
	}
	
	private File pathToFile(String path){
		return new File(new File(root, "data"), path);
	}
	
	private File pathToConfig(String path){
		int dotIndex = path.lastIndexOf(".");
		if(dotIndex > 0){
			path = path.substring(0, dotIndex);
		}
		String xmlPath = path + ".xml";
		return new File(new File(root, "conf"), xmlPath);
	}
	
	private void executeActions(String path, VelocityContext context) throws Exception{
		File configFile = pathToConfig(path);
		if(!configFile.exists()){
			return;
		}
		
		FileReader reader = null;
		ActionManager actionManager = siteContext.getActionManager();
		
		try{
			reader = new FileReader(configFile);
			SAXBuilder builder = new SAXBuilder();
			Document document = builder.build(reader);
			Element rootElement = document.getRootElement();
			
			Iterator contentActionElements = rootElement.getChildren("content-action", NS).iterator();
			while(contentActionElements.hasNext()){
				Element contentActionElement = (Element)contentActionElements.next();
				actionManager.execute(contentActionElement.getAttributeValue("name"), context);
			}
		} finally {
			IOUtilities.close(reader);
		}
	}
	
	private static Logger log = LogManager.getLogger(ExtendedFileSystemRepository.class.getName());
	
	private String root;
	private String dataPath;
	private String configPath;
	private String name;

}
