/*-- 

 Copyright (C) 2001 Anthony Eden.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions, and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions, and the disclaimer that follows 
    these conditions in the documentation and/or other materials 
    provided with the distribution.

 3. The name "JPublish" must not be used to endorse or promote products
    derived from this software without prior written permission.  For
    written permission, please contact me@anthonyeden.com.
 
 4. Products derived from this software may not be called "JPublish", nor
    may "JPublish" appear in their name, without prior written permission
    from Anthony Eden (me@anthonyeden.com).
 
 In addition, I request (but do not require) that you include in the 
 end-user documentation provided with the redistribution and/or in the 
 software itself an acknowledgement equivalent to the following:
     "This product includes software developed by
      Anthony Eden (http://www.anthonyeden.com/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.

 For more information on JPublish, please see <http://www.jpublish.org/>.
 
 */

package org.jpublish.repository.filesystem;

import java.io.File;
import java.io.FileReader;
import java.io.StringWriter;

import org.jdom.Element;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

import com.anthonyeden.lib.log.Logger;
import com.anthonyeden.lib.log.LogManager;
import com.anthonyeden.lib.util.IOUtilities;

import org.jpublish.Configurable;
import org.jpublish.repository.AbstractRepository;

/**	An implementation of the Repository interface which pulls content from
	the local file system.

	@author Anthony Eden
*/

public class FileSystemRepository extends AbstractRepository implements Configurable{

	/**	Get the content from the given path.  Implementations of this method
		should NOT merge the content using Velocity.

		@param path The relative content path
		@return The content as a String
		@throws Exception Any Exception
	*/

	public String get(String path) throws Exception{
		log.debug("Getting static content element.");
		return loadContent(path);
	}
	
	/**	Get the content from the given path and merge it with
		the given context.
		
		@param path The content path
		@param context The VelocityContext
		@return The content as a String
		@throws Exception Any Exception
	*/

	public String get(String path, VelocityContext context) throws Exception{
		log.debug("Getting dynamic content element for path " + path);
		
		StringWriter writer = null;
		FileReader reader = null;
		
		try{
			reader = new FileReader(pathToFile(path));
			writer = new StringWriter();
			
			Velocity.evaluate(context, writer, path, reader);
			String content = writer.toString();
			return content;
		} finally {
			IOUtilities.close(reader);
			IOUtilities.close(writer);
		}
	}

	/**	Get the name of the repository.  This name is used to expose the
		Repository in the Velocity engine.

		@return The Repository name
	*/

	public String getName(){
		return name;
	}
	
	/**	Get the last modified time in milliseconds for the given path.
	
		@param path The content path
		@return The last modified time in milliseconds
		@throws Exception Any Exception
	*/
	
	public long getLastModified(String path) throws Exception{
		return pathToFile(path).lastModified();
	}

	/**	Get the root directory for locating content.

		@return The root directory
	*/

	public String getRoot(){
		return root;
	}

	/**	Set the root directory for locating content.

		@param root The new root directory
	*/

	public void setRoot(String root){
		this.root = root;
	}
	
	/**	Load the repository's configuration from the given JDOM element.
	
		@param element The JDOM element
		@throws Exception
	*/
	
	public void loadConfiguration(Element element) throws Exception{
		this.name = element.getAttributeValue("name");
		setRoot(element.getChildTextTrim("root"));
	}
	
	/**	Load the content from the given path.
	
		@param path The path
		@return The String
	*/
	
	private String loadContent(String path){
		StringBuffer buffer = new StringBuffer();
		FileReader reader = null;
		
		try{
			reader = new FileReader(pathToFile(path));
			
			int c = -1;
			while((c = reader.read()) != -1){
				buffer.append((char)c);
			}
		} catch(Exception e){
			buffer.append(e.getMessage());
		} finally {
			IOUtilities.close(reader);
		}
		
		return buffer.toString();
	}
	
	/**	Convert a path to a file.
	
		@param path The path
		@return The file
	*/
	
	private File pathToFile(String path){
		return new File(root, path);
	}
	
	private static Logger log = LogManager.getLogger(FileSystemRepository.class.getName());
	
	private String root;
	private String name;

}
