/*-- 

 Copyright (C) 2001 Anthony Eden.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions, and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions, and the disclaimer that follows 
    these conditions in the documentation and/or other materials 
    provided with the distribution.

 3. The name "JPublish" must not be used to endorse or promote products
    derived from this software without prior written permission.  For
    written permission, please contact me@anthonyeden.com.
 
 4. Products derived from this software may not be called "JPublish", nor
    may "JPublish" appear in their name, without prior written permission
    from Anthony Eden (me@anthonyeden.com).
 
 In addition, I request (but do not require) that you include in the 
 end-user documentation provided with the redistribution and/or in the 
 software itself an acknowledgement equivalent to the following:
     "This product includes software developed by
      Anthony Eden (http://www.anthonyeden.com/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.

 For more information on JPublish, please see <http://www.jpublish.org/>.
 
 */

package org.jpublish.util;

import java.net.URLEncoder;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.anthonyeden.lib.log.Logger;
import com.anthonyeden.lib.log.LogManager;

public class URLUtilities{

	public URLUtilities(HttpServletRequest request, HttpServletResponse response){
		this.request = request;
		this.response = response;
	}
	
	/**	Build an HTTP URL relative to the application context using the given
		path.
		
		@param path The path
	*/

	public String buildStandard(String path){
		return buildStandard(path, 0);
	}
	
	/**	Build an HTTP URL relative to the application context using the given path.  
		This version of the <code>buildStandard</code> method allows you to specify 
		the port number.  A port number of 0 will cause the port argument to be ignored.
		
		@param path The path
		@param port The port
	*/
	
	public String buildStandard(String path, int port){
		return build(path, "http", port);
	}
	
	/**	Build an HTTPS (Secure Socket Layer) method relative to the application context 
		using the given path.
		
		@param path The path
	*/

	public String buildSecure(String path){
		return buildSecure(path, 0);
	}
	
	/**	Build an HTTPS (Secure Socket Layer) method relative to the application context 
		using the given path.  This version of the <code>buildSecure</code> method 
		allows you to specify the port number.  A port number of 0 will cause the port 
		argument to be ignored.
		
		@param path The path
		@param port The port
	*/
	
	public String buildSecure(String path, int port){
		return build(path, "https", port);
	}
	
	/**	Build a URL using the given path, protocol and port.  The path will be relative
		to the current context.
		
		@param path The path
		@param protocol (i.e. http or https)
		@param port The port (0 to ignore the port argument)
		@return The URL as a String
	*/
	
	protected String build(String path, String protocol, int port){
		String serverName = request.getServerName();
		String contextPath = request.getContextPath();
		
		log.debug("Server name: " + serverName);
		log.debug("Context path: " + contextPath);
		
		if(!contextPath.endsWith(URL_PATH_SEPARATOR)){
			contextPath = contextPath + URL_PATH_SEPARATOR;
		}
		
		if(path.startsWith(URL_PATH_SEPARATOR)){
			path = path.substring(1);
		}
		
		String requestPath = contextPath + path;
		log.debug("Request path: " + requestPath);
		
		StringBuffer buffer = new StringBuffer();
		buffer.append(protocol);
		buffer.append("://");
		buffer.append(serverName);
		
		if(port > 0){
			buffer.append(":");
			buffer.append(port);
		}
		
		if(!requestPath.startsWith(URL_PATH_SEPARATOR)){
			buffer.append(URL_PATH_SEPARATOR);
		}
		
		buffer.append(requestPath);
		
		log.debug("URL: '" + buffer + "'");
		
		return buffer.toString();
	}
	
	/**	Percent-encode the given String.  This method delegates to
		the URLEncoder.encode() method.
	
		@param s The String to encode
		@return The encoded String
		@see java.net.URLEncoder
	*/
	
	public String encode(String s){
		return URLEncoder.encode(s);
	}
	
	/**	The URL path separator. */
	
	public static final String URL_PATH_SEPARATOR = "/";
	
	private static Logger log = LogManager.getLogger(URLUtilities.class.getName());
	
	private HttpServletRequest request;
	private HttpServletResponse response;

}
